% num = match(image1, image2)
% Double consistency check
% This function reads two images, finds their SIFT features, and
%   displays lines connecting the matched keypoints.  A match is accepted
%   only if its distance is less than distRatio times the distance to the
%   second closest match.
% It returns the number of matches displayed.
% 
%
% Example: match2check('scene.pgm','book.pgm');

function [ind1, ind2] = match2check(des1, des2, distRatio)

% For efficiency in Matlab, it is cheaper to compute dot products between
%  unit vectors rather than Euclidean distances.  Note that the ratio of 
%  angles (acos of dot products of unit vectors) is a close approximation
%  to the ratio of Euclidean distances for small angles.
%
% distRatio: Only keep matches in which the ratio of vector angles from the
%   nearest to second nearest neighbor is less than distRatio.
%distRatio = 0.8;   

des1 = des1/sqrt(3);
des2 = des2/sqrt(3);

% For each descriptor in the first image, select its match to second image.
des2t = des2';                          % Precompute matrix transpose
for i = 1 : size(des1,1)
   dotprods = des1(i,:) * des2t;        % Computes vector of dot products
   [vals,indx] = sort(acos(dotprods));  % Take inverse cosine and sort results

   % Check if nearest neighbor has angle less than distRatio times 2nd.
   if (vals(1) < distRatio * vals(2))
      match1(i) = indx(1);
   else
      match1(i) = 0;
   end
end

des1t=des1';
for i = 1 : size(des2,1)
   dotprods = des2(i,:) * des1t;        % Computes vector of dot products
   [vals,indx] = sort(acos(dotprods));  % Take inverse cosine and sort results

   % Check if nearest neighbor has angle less than distRatio times 2nd.
   if (vals(1) < distRatio * vals(2))
      match2(i) = indx(1);
   else
      match2(i) = 0;
   end
end

% Finds left/right matches
num=0;
matches=zeros(size(match1));
for i = 1: size(des1,1)
  if (match1(i) > 0)
      if (match2(match1(i)) == i)
        matches(i)=match1(i);
        num=num+1;
     end
  end
end
%fprintf('Found %d left/right matches.\n', num);

ind1 = find(matches);
ind2 = matches(ind1);